@@ -1,4 +1,3 @@
-import string
 import re
 
 Zero = "PyUFunc_Zero"
@@ -6,6 +5,16 @@ One = "PyUFunc_One"
 None_ = "PyUFunc_None"
 
 class TypeDescription(object):
+    """Type signature for a ufunc
+
+    Attributes
+    ----------
+
+    type: character representing the type
+    func_data:
+    in_:
+    out:
+    """
     def __init__(self, type, f=None, in_=None, out=None):
         self.type = type
         self.func_data = f
@@ -56,6 +65,17 @@ def TD(types, f=None, in_=None, out=None):
     return tds
 
 class Ufunc(object):
+    """Description of a ufunc.
+
+    Attributes
+    ----------
+
+    nin: number of input arguments
+    nout: number of output arguments
+    identity: identity element for a two-argument function
+    docstring: docstring for the ufunc
+    type_descriptions: list of TypeDescription objects
+    """
     def __init__(self, nin, nout, identity, docstring,
                  *type_descriptions):
         self.nin = nin
@@ -70,7 +90,7 @@ class Ufunc(object):
         for td in self.type_descriptions:
             td.finish_signature(self.nin, self.nout)
 
-#each entry in defdict is
+#each entry in defdict is a Ufunc object.
 
 #name: [string of chars for which it is defined,
 #       string of characters using func interface,
@@ -81,6 +101,29 @@ class Ufunc(object):
 #       output specification (optional)
 #       ]
 
+chartoname = {'?': 'bool',
+              'b': 'byte',
+              'B': 'ubyte',
+              'h': 'short',
+              'H': 'ushort',
+              'i': 'int',
+              'I': 'uint',
+              'l': 'long',
+              'L': 'ulong',
+              'q': 'longlong',
+              'Q': 'ulonglong',
+              'f': 'float',
+              'd': 'double',
+              'g': 'longdouble',
+              'F': 'cfloat',
+              'D': 'cdouble',
+              'G': 'clongdouble',
+              'O': 'OBJECT',
+              # M is like O, but calls a method of the object instead
+              # of a function
+              'M': 'OBJECT',
+              }
+
 all = '?bBhHiIlLqQfdgFDGO'
 O = 'O'
 M = 'M'
@@ -237,25 +280,25 @@ defdict = {
     Ufunc(2, 1, One,
           'returns x1 and x2 elementwise.',
           TD(noobj, out='?'),
-          TD(M, f='logical_and', out='?'),
+          TD(M, f='logical_and'),
           ),
 'logical_not' :
     Ufunc(1, 1, None,
           'returns not x elementwise.',
           TD(noobj, out='?'),
-          TD(M, f='logical_not', out='?'),
+          TD(M, f='logical_not'),
           ),
 'logical_or' :
     Ufunc(2, 1, Zero,
           'returns x1 or x2 elementwise.',
           TD(noobj, out='?'),
-          TD(M, f='logical_or', out='?'),
+          TD(M, f='logical_or'),
           ),
 'logical_xor' :
     Ufunc(2, 1, None,
           'returns x1 xor x2 elementwise.',
           TD(noobj, out='?'),
-          TD(M, f='logical_xor', out='?'),
+          TD(M, f='logical_xor'),
           ),
 'maximum' :
     Ufunc(2, 1, None,
@@ -305,6 +348,16 @@ defdict = {
           TD(ints),
           TD(O, f='PyNumber_Rshift'),
           ),
+'degrees' :
+    Ufunc(1, 1, None,
+          'converts angle from radians to degrees',
+          TD(fltsM, f='degrees'),
+          ),
+'radians' :
+    Ufunc(1, 1, None,
+          'converts angle from degrees to radians',
+          TD(fltsM, f='radians'),
+          ),
 'arccos' :
     Ufunc(1, 1, None,
           'inverse cosine elementwise.',
@@ -484,32 +537,11 @@ defdict = {
 
 def indent(st,spaces):
     indention = ' '*spaces
-    indented = indention + string.replace(st,'\n','\n'+indention)
+    indented = indention + st.replace('\n','\n'+indention)
     # trim off any trailing spaces
     indented = re.sub(r' +$',r'',indented)
     return indented
 
-chartoname = {'?': 'bool',
-              'b': 'byte',
-              'B': 'ubyte',
-              'h': 'short',
-              'H': 'ushort',
-              'i': 'int',
-              'I': 'uint',
-              'l': 'long',
-              'L': 'ulong',
-              'q': 'longlong',
-              'Q': 'ulonglong',
-              'f': 'float',
-              'd': 'double',
-              'g': 'longdouble',
-              'F': 'cfloat',
-              'D': 'cdouble',
-              'G': 'clongdouble',
-              'O': 'OBJECT',
-              'M': 'OBJECT',
-              }
-
 chartotype1 = {'f': 'f_f',
                'd': 'd_d',
                'g': 'g_g',
@@ -532,6 +564,39 @@ chartotype2 = {'f': 'ff_f',
 # 2) fill in functions and data in InitOperators
 # 3) add function.
 
+# String-handling utilities to avoid locale-dependence.
+
+import string
+UPPER_TABLE = string.maketrans(string.ascii_lowercase, string.ascii_uppercase)
+
+def english_upper(s):
+    """ Apply English case rules to convert ASCII strings to all upper case.
+
+    This is an internal utility function to replace calls to str.upper() such
+    that we can avoid changing behavior with changing locales. In particular,
+    Turkish has distinct dotted and dotless variants of the Latin letter "I" in
+    both lowercase and uppercase. Thus, "i".upper() != "I" in a "tr" locale.
+
+    Parameters
+    ----------
+    s : str
+
+    Returns
+    -------
+    uppered : str
+
+    Examples
+    --------
+    >>> from numpy.lib.utils import english_upper
+    >>> english_upper('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_')
+    'ABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_'
+    >>> english_upper('')
+    ''
+    """
+    uppered = s.translate(UPPER_TABLE)
+    return uppered
+
+
 def make_arrays(funcdict):
     # functions array contains an entry for every type implemented
     #   NULL should be placed where PyUfunc_ style function will be filled in later
@@ -568,15 +633,19 @@ def make_arrays(funcdict):
                 elif t.type == 'M':
                     datalist.append('(void *)"%s"' % t.func_data)
                 else:
-                    datalist.append('(void *)%s' % t.func_data)
+                    astr = '%s_data[%d] = (void *) %s;' % \
+                           (name, k, t.func_data)
+                    code2list.append(astr)
+                    datalist.append('(void *)NULL')
+                    #datalist.append('(void *)%s' % t.func_data)
                 sub += 1
             else:
                 datalist.append('(void *)NULL');
-                tname = chartoname[t.type].upper()
+                tname = english_upper(chartoname[t.type])
                 funclist.append('%s_%s' % (tname, name))
 
             for x in t.in_ + t.out:
-                siglist.append('PyArray_%s' % (chartoname[x].upper(),))
+                siglist.append('PyArray_%s' % (english_upper(chartoname[x]),))
 
             k += 1
 
