@@ -792,7 +792,7 @@ class TestMA(NumpyTestCase):
         series = data[[0,2,1]]
         assert_equal(series._fill_value, data._fill_value)
         #
-        mtype = [('f',float_),('s','|S3')]
+        mtype = [('f',float),('s','|S3')]
         x = array([(1,'a'),(2,'b'),(numpy.pi,'pi')], dtype=mtype)
         x.fill_value=999
         assert_equal(x.fill_value,[999.,'999'])
@@ -865,6 +865,31 @@ class TestMA(NumpyTestCase):
         assert_equal(xs.dtype, '|S3')
 
 
+    def test_reshape(self):
+        a = arange(10)
+        a[0] = masked
+        # Try the default
+        b = a.reshape((5,2))
+        assert_equal(b.shape, (5,2))
+        assert(b.flags['C'])
+        # Try w/ arguments as list instead of tuple
+        b = a.reshape(5,2)
+        assert_equal(b.shape, (5,2))
+        assert(b.flags['C'])
+        # Try w/ order
+        b = a.reshape((5,2), order='F')
+        assert_equal(b.shape, (5,2))
+        assert(b.flags['F'])
+        # Try w/ order
+        b = a.reshape(5,2, order='F')
+        assert_equal(b.shape, (5,2))
+        assert(b.flags['F'])
+        #
+        c = np.reshape(a, (2,5))
+        assert(isinstance(c, MaskedArray))
+        assert_equal(c.shape, (2,5))
+        assert(c[0,0] is masked)
+        assert(c.flags['C'])
 
 #...............................................................................
 
@@ -1276,7 +1301,7 @@ class TestArrayMethods(NumpyTestCase):
         #
         x = [1,4,2,3]
         sortedx = sort(x)
-        assert(not isinstance(sorted, MaskedArray))
+        assert(not isinstance(sortedx, MaskedArray))
         #
         x = array([0,1,-1,-2,2], mask=nomask, dtype=numpy.int8)
         sortedx = sort(x, endwith=False)
@@ -1584,6 +1609,25 @@ class TestMiscFunctions(NumpyTestCase):
         assert_almost_equal(x._data,y._data)
 
 
+    def test_divide_on_different_shapes(self):
+        x = arange(6, dtype=float)
+        x.shape = (2,3)
+        y = arange(3, dtype=float)
+        #
+        z = x/y
+        assert_equal(z, [[-1.,1.,1.], [-1.,4.,2.5]])
+        assert_equal(z.mask, [[1,0,0],[1,0,0]])
+        #
+        z = x/y[None,:]
+        assert_equal(z, [[-1.,1.,1.], [-1.,4.,2.5]])
+        assert_equal(z.mask, [[1,0,0],[1,0,0]])
+        #
+        y = arange(2, dtype=float)
+        z = x/y[:,None]
+        assert_equal(z, [[-1.,-1.,-1.], [3.,4.,5.]])
+        assert_equal(z.mask, [[1,1,1],[0,0,0]])
+
+
 
 ###############################################################################
 #------------------------------------------------------------------------------
