@@ -389,7 +389,7 @@ long rk_binomial_inversion(rk_state *state, long n, double p)
         state->q = q = 1.0 - p;
         state->r = qn = exp(n * log(q));
         state->c = np = n*p;
-        state->m = bound = min(n, np + 10.0*sqrt(np));
+        state->m = bound = min(n, np + 10.0*sqrt(np*q + 1));
     } else
     {
         q = state->q;
@@ -676,16 +676,23 @@ long rk_zipf(rk_state *state, double a)
 {
     double T, U, V;
     long X;
-    double b;
-    
-    b = pow(2.0, a-1.0);
+    double am1, b;
+
+    am1 = a - 1.0;
+    b = pow(2.0, am1);
     do
     {
-        U = rk_double(state);
+        U = 1.0-rk_double(state);
         V = rk_double(state);
-        X = (long)floor(pow(U, -1.0/(a-1.0)));
-        T = pow(1.0 + 1.0/X, a-1.0);  
-    } while ((V *X*(T-1.0)/(b-1.0)) > (T/b));
+        X = (long)floor(pow(U, -1.0/am1));
+        /* The real result may be above what can be represented in a signed
+         * long. It will get casted to -sys.maxint-1. Since this is
+         * a straightforward rejection algorithm, we can just reject this value
+         * in the rejection condition below. This function then models a Zipf
+         * distribution truncated to sys.maxint.
+         */
+        T = pow(1.0 + 1.0/X, am1);
+    } while (((V*X*(T-1.0)/(b-1.0)) > (T/b)) || X < 1);
     return X;
 }
 
@@ -742,7 +749,7 @@ long rk_hypergeometric_hyp(rk_state *state, long good, long bad, long sample)
         if (K == 0) break;
     }
     Z = (long)(d2 - Y);
-    if (bad > good) Z = sample - Z;
+    if (good > bad) Z = sample - Z;
     return Z;
 }
 
@@ -795,7 +802,7 @@ long rk_hypergeometric_hrua(rk_state *state, long good, long bad, long sample)
     }
     
     /* this is a correction to HRUA* by Ivan Frohne in rv.py */
-    if (bad > good) Z = m - Z;
+    if (good > bad) Z = m - Z;
     
     /* another fix from rv.py to allow sample to exceed popsize/2 */
     if (m < sample) Z = bad - Z;
